#include <avr/io.h>
#include <util/delay.h>

#include "bmp180.h"
#include "i2cSoft.h"

//#include "uart.h" // 
//#include <stdlib.h>	 // 

typedef struct {
	int16_t ac1;//0xAA 0xAB
	int16_t ac2;//0xAC 0xAD
	int16_t ac3;//0xAE 0xAF
	uint16_t ac4;
	uint16_t ac5;
	uint16_t ac6;
	int16_t b1;
	int16_t b2;
	int16_t mb;
	int16_t mc;
	int16_t md;
	} BMP180Eeprom;
	
typedef union{
	uint8_t u8[4];
	uint16_t u16;
	uint32_t u32;
	int16_t i16;
	int32_t i32;
	} BmpP; 

typedef union{
	uint8_t u8[2];
	uint16_t u16;
	int16_t i16;
	} BmpT; 	

BMP180Eeprom S;
BmpP BmpPress; 
BmpT BmpTemper;

int32_t Bmp180Pressure; // 
int16_t Bmp180Temper; // 0,1
uint16_t Rt_mm;//   
uint8_t FlagBMP180;

//int32_t altitude; // 


//-----------------------------
uint8_t bmp180_read_calibration(void);
void measure_bmp180(void);
//------------------------------------

uint8_t bmp180_read_calibration(void){
uint8_t error=0;
uint8_t Ts[sizeof(S)];	

/*
	for(uint8_t i=0; i<sizeof(S); i++){
		error=i2c_Start();//start
		error=i2c_wr_byte(BMP180_I2C_ADR_W); //write
		error=i2c_wr_byte(BMP180_ADR_CALIBRATION+i); //adress
		i2c_Stop();
		error=i2c_Start();//restart
		error=i2c_wr_byte(BMP180_I2C_ADR_R); //read
		Ts[i]=i2c_rx_byte(0);//no ASK
		i2c_Stop();
		}

	uint8_t *p;//      	
	p=(uint8_t *)&S;
	for(uint8_t i=0; i<sizeof(S); i+=2){
		p[i]=Ts[i+1];
		p[i+1]=Ts[i];
		}
		
//*/
		
//--------------------------
//*	
	error=i2c_Start();
	error=i2c_wr_byte(BMP180_I2C_ADR_W); 
	error=i2c_wr_byte(BMP180_ADR_CALIBRATION); 
	error=i2c_Stop();
	error=i2c_Start();
	error=i2c_wr_byte(BMP180_I2C_ADR_R); 
	
	for(uint8_t i=0; i<sizeof(Ts)-1; i++){
			Ts[i]=i2c_rx_byte(1);//ASK
		}
	Ts[sizeof(Ts)-1]=i2c_rx_byte(0);//no ASK	
	error=i2c_Stop();
	
	uint8_t *p;
	p=(uint8_t *)&S;
	for(uint8_t i=0; i<sizeof(S); i+=2){
		p[i]=Ts[i+1];//      	
		p[i+1]=Ts[i];
		}

//	*/	
//--------------------------	

	return error;
}

/*
//       () (     -   !!!)
static void bmp180CalcAltitude(int32_t pressure){
	float temp;

	temp = (float) pressure/101325;
	temp = 1-pow(temp, 0.19029);
	//altitude = round(44330*temp*10);
	altitude = 44330*temp*100;
	//get altitude in dm
}
*/
//-------------------------------------------------------------

void measure_bmp180(void){//  1
static uint8_t State, Time1ms;
uint8_t error=0;

	switch(State){                     
		case (0):
			if (--Time1ms==0) State=1;
			break;
		
		case (1):
			error=i2c_Start();
			error|=i2c_wr_byte(BMP180_I2C_ADR_W);
			error|=i2c_wr_byte(BMP180_COMMAND_SOFT_RESET);
			error|=i2c_wr_byte(BMP180_COMMAND_SOFT_RESET_DATA);
			error|=i2c_Stop();
			Time1ms=100;
			State=2;
			break;

		case (2):
			if (--Time1ms==0) State=3;
			break;		
	
		case (3):
			error=bmp180_read_calibration();
			State=7;
			break;
		case (7):
			if (FlagBMP180 & FL_BMP180_MEASURE_END)	break;//,     
			FlagBMP180|=FL_BMP180_T_MEASURE;				
		case (8): 
			error=i2c_Start();
			error|=i2c_wr_byte(BMP180_I2C_ADR_W); 
			error|=i2c_wr_byte(BMP180_REG_CONTROL); 
			Time1ms=100;
			uint8_t Cmd=BMP180_COMMAND_PRESSURE;
			if ( FlagBMP180 & FL_BMP180_T_MEASURE ) {
				Time1ms=7;
				Cmd=BMP180_COMMAND_TEMPERATURE;
				}
			error|=i2c_wr_byte(Cmd);
			error|=i2c_Stop();
			State=9;
			break;
		
		case (9):
			if (--Time1ms==0) State=10;
			break;
			
		case (10): 
			{
			uint8_t Buf[3];
			//S CMD_WRITE REG_ADR P S CMD_READ R P
			//Buf[0] MSB	adr=0xF6
			//Buf[1] LSB	adr=0xF7
			//Buf[2] XLSB	adr=0xF7
			error=i2c_Start();
			error|=i2c_wr_byte(BMP180_I2C_ADR_W);
			error|=i2c_wr_byte(BMP180_REG_RESULT);
			error|=i2c_Stop();
			error|=i2c_Start();
			error|=i2c_wr_byte(BMP180_I2C_ADR_R);
			Buf[0]=i2c_rx_byte(1);	//ASK
			if ( FlagBMP180 & FL_BMP180_T_MEASURE )
				Buf[1]=i2c_rx_byte(0);	//NASK
				else{
				Buf[1]=i2c_rx_byte(1);	//ASK
				Buf[2]=i2c_rx_byte(0);	//NASK
				}
			error|=i2c_Stop();

			if ( FlagBMP180 & FL_BMP180_T_MEASURE ){ 
				FlagBMP180&=~FL_BMP180_T_MEASURE;
				BmpTemper.u8[0]=Buf[1];
				BmpTemper.u8[1]=Buf[0];			
				State=8; 
				}else{
				BmpPress.u8[0]=Buf[2];
				BmpPress.u8[1]=Buf[1];
				BmpPress.u8[2]=Buf[0];				
				BmpPress.i32=BmpPress.i32>>(8-OSS);			
				State=11;
				}
			}
			break;

		case (11): 
			{
			int32_t ut=BmpTemper.i16;
			int32_t up=BmpPress.i32;
			int32_t x1, x2, b5, b6, x3, b3, p;
			uint32_t b4, b7;

			//
			x1 = ( ((int32_t)ut - (int32_t)S.ac6) * (int32_t)S.ac5 ) >> 15;
			x2 = ((int32_t)S.mc << 11) / (x1 + S.md);
			b5 = x1 + x2;
			Bmp180Temper = (b5 + 8) >> 4;
			
			//
			b6 = b5 - 4000;
			x1 = (S.b2 * ((b6 * b6) >> 12)) >> 11;
			x2 = ((int32_t)S.ac2 * b6) >> 11;
			x3 = x1 + x2;
			b3 = ((( (int32_t)S.ac1 * 4 + x3 )<<OSS) + 2)/4;
			x1 = ((int32_t)S.ac3 * b6) >> 13;
			x2 = ((int32_t)S.b1 * ((b6 * b6) >> 12)) >> 16;
			x3 = ((x1 + x2) + 2) >> 2;
			b4 = ((uint32_t)S.ac4 * (uint32_t) (x3 + 32768)) >> 15;
			b7 = ((uint32_t) (up - b3) * (50000 >> OSS));
	
			if (b7 < 0x80000000)
			{
				p = (b7 << 1) / b4;
			}
			else
			{ 
				p = (b7 / b4) << 1;
			}

			x1 = (p >> 8) * (p >> 8);
			x1 = (x1 * 3038) >> 16;
			x2 = (-7357 * p) >> 16;
			Bmp180Pressure = p + ((x1 + x2 + 3791) >> 4);
			
			Rt_mm=(Bmp180Pressure*7600)/101325;// 7600= 760 * 10
			
			FlagBMP180|=FL_BMP180_MEASURE_END;
			FlagBMP180 &= ~ (FL_BMP180_ERROR|FL_BMP180_NO_SENSOR);
			State=7;
			break;
			}
		/*	
		case (12): 
			{
			bmp180CalcAltitude(Bmp180Pressure);
			uart_puts(" altitude=");
			ltoa(altitude, buf, 10);
			uart_puts(buf);
			
			State=7;
			break;
			}*/
			
		default : 
			Time1ms=20;
			State=0;
			break;	
		}
	
	if (error){
		FlagBMP180|=FL_BMP180_ERROR|FL_BMP180_NO_SENSOR;
		State=0xFF;
		}
}


