#include "sht10.h"
                           //adr  command  r/w
#define STATUS_REG_W 0x06   //000   0011    0
#define STATUS_REG_R 0x07   //000   0011    1
#define MEASURE_TEMP 0x03   //000   0001    1
#define MEASURE_HUMI 0x05   //000   0010    1
#define RESET        0x1e   //000   1111    0



//-------------------
#define	SCK_DOWN	OFF(SCK)//DRIVER(SCK,OUT)
#define	SCK_UP		ON(SCK)//DRIVER(SCK,IN)
#define	DATA_DOWN	DRIVER(DATA,OUT)
#define	DATA_UP		DRIVER(DATA,IN)

typedef union 
	{
	uint8_t Byte[2];
	uint16_t i;
	float f;
	} Shtvalue;
Shtvalue TempSht, HumiSht;
int ShtHumi, ShtTemper;

//----------------------SHT10----------------------------------
static void sht_sart(void){
// generates a transmission start 
//       _____         ________
// DATA:      |_______|
//           ___     ___
// SCK : ___|   |___|   |______
	OFF(DATA);
	//DRIVER(SCK,OUT);
	OFF(SCK);
	
	DATA_UP;
	_delay_us(3);
	ON(SCK);
	_delay_us(3);
	DATA_DOWN;
	_delay_us(3);
	OFF(SCK);
	_delay_us(3);
	ON(SCK);
	_delay_us(3);
	DATA_UP;
	_delay_us(3);
	OFF(SCK);
}

static uint8_t sht_wr_byte(uint8_t Byte){
	
	DATA_UP;
	_delay_us(3);
	
	if (ACTIVE(DATA)==0) return 1;
	
	for(uint8_t b=_BV(7); b; b>>=1){
		if (Byte & b) 	DATA_UP; else DATA_DOWN;
		_delay_us(3);
		ON(SCK);
		_delay_us(5);
		OFF(SCK);
		_delay_us(3);
		}
	DATA_UP;	
	//----ASK----
	uint8_t Ask=0;
	ON(SCK);
	_delay_us(3);
	if (ACTIVE(DATA)) Ask=1;//   
	OFF(SCK);
	
	return Ask;
}

static uint8_t sht_rx_byte(uint8_t Ask){
uint8_t Val=0;
	
	for(uint8_t b=_BV(7); b; b>>=1){
		ON(SCK);
		_delay_us(3);
		if (ACTIVE(DATA))	Val|=b;
		OFF(SCK);
		_delay_us(3);
		}
	
	if (Ask) DATA_DOWN;
	ON(SCK);
	_delay_us(5);
	OFF(SCK);
	DATA_UP;
	return Val;
}

//----------------------------------------------------------------------------------
static void s_connectionreset(void){
//----------------------------------------------------------------------------------
// communication reset: DATA-line=1 and at least 9 SCK cycles followed by transstart
//       _____________________________________________________         ________
// DATA:                                                      |_______|
//          _    _    _    _    _    _    _    _    _        ___     ___
// SCK : __| |__| |__| |__| |__| |__| |__| |__| |__| |______|   |___|   |______  
	DATA_UP; 
	OFF(SCK);                    //Initial state
	for(uint8_t i=9;i;i--){                  //9 SCK cycles
		ON(SCK);
		_delay_us(3);
		OFF(SCK);
		_delay_us(3);
		}
	sht_sart();                   //transmission start
}
//----------------------------------------------------------------------------------
static uint8_t s_softreset(void){// resets the sensor by a softreset 
  s_connectionreset();              //reset communication
  return sht_wr_byte(RESET);       //send RESET-command to sensor                    
}
 
//----------------------------------------------------------------------------------------
static void calc_sth11(float *p_humidity ,float *p_temperature)
//----------------------------------------------------------------------------------------
// calculates temperature [C] and humidity [%RH] 
// input :  humi [Ticks] (12 bit) 
//          temp [Ticks] (14 bit)
// output:  humi [%RH]
//          temp [C]
{ const float C1=-2.0468;           // for 12 Bit RH
  const float C2=+0.0367;           // for 12 Bit RH
  const float C3=-0.0000015955;     // for 12 Bit RH
  const float T1=+0.01;             // for 12 Bit RH
  const float T2=+0.00008;          // for 12 Bit RH	

  float rh=*p_humidity;             // rh:      Humidity [Ticks] 12 Bit 
  float t=*p_temperature;           // t:       Temperature [Ticks] 14 Bit
  float rh_lin;                     // rh_lin:  Humidity linear
  float rh_true;                    // rh_true: Temperature compensated humidity
  float t_C;                        // t_C   :  Temperature [C]

  t_C=t*0.01 - 40.1;                //calc. temperature [C] from 14 bit temp. ticks @ 5V
  rh_lin=C3*rh*rh + C2*rh + C1;     //calc. humidity from ticks to [%RH]
  rh_true=(t_C-25)*(T1+T2*rh)+rh_lin;   //calc. temperature compensated humidity [%RH]
  if(rh_true>100)rh_true=100;       //cut if the value is outside of
  if(rh_true<0.1)rh_true=0.1;       //the physical possible range

  *p_temperature=t_C;               //return temperature [C]
  *p_humidity=rh_true;              //return humidity[%RH]
  //*p_humidity=rh_lin;              
}



/*//--------------------------------------------------------------------
static float calc_dewpoint(float h,float t)
//--------------------------------------------------------------------
// calculates dew point
// input:   humidity [%RH], temperature [C]
// output:  dew point [C]
{ float k,dew_point ;
  k = (log10(h)-2)/0.4343 + (17.62*t)/(243.12+t);
  dew_point = 243.12*k/(17.62-k);
  return dew_point;
}*/

//--------------------------------------------------------------
uint8_t measure_sht(void){
static uint8_t State, Time1ms;
static uint16_t TimeError;
uint8_t error=0;

	switch(State){                     
		case (0):
			if (--Time1ms==0) State=1;
			break;
		case (1):
			sht_sart();
			error=sht_wr_byte(MEASURE_TEMP); 
			TimeError=0;
			State=2;
			break;
		case (2): 
			if ( (ACTIVE(DATA))==0 ) State=3;
			if ( ++TimeError==1000 ) State=0xFF;
			break;
		case (3): 
			TempSht.Byte[1]=sht_rx_byte(1);
			TempSht.Byte[0]=sht_rx_byte(0);//stop connect (no ask)
			State=4;
			break;	
		case (4):
			sht_sart();
			error=sht_wr_byte(MEASURE_HUMI); 
			TimeError=0;
			State=5;
			break;
		case (5): 
			if ( (ACTIVE(DATA))==0 ) State=6;
			if ( ++TimeError==1000 ) State=0xFF;
			break;
		case (6): 
			HumiSht.Byte[1]=sht_rx_byte(1);
			HumiSht.Byte[0]=sht_rx_byte(0);//stop connect (no ask)
			State=7;
			break;
		case (7): 
			TempSht.f=(float)TempSht.i;
			HumiSht.f=(float)HumiSht.i;
			State=8;
			break;

		case (8): 
			calc_sth11(&HumiSht.f,&TempSht.f);
			ShtHumi=(int)((HumiSht.f)*10);
			ShtTemper=(int)((TempSht.f)*10);
			State=9;
			break;
		case (9): 
			State=1;
			return 1;
			break;
		
		default : 
			s_softreset();
			Time1ms=20;
			State=0;
			break;	 
		}
	if (error){
		State=0xFF;
		return 0xFF;
		}
	
	return 0;
}

//---------------------END-SHT10----------------------------------
