#include <avr/io.h>
#include "sht21.h" 
#include "i2cSoft.h"

//#include "uart.h" // 
//#include <stdlib.h>	 // 

//-------------------SHT21-----------------------------
//  CRC
//const uint16_t POLYNOMIAL = 0x131;  //P(x)=x^8+x^5+x^4+1 = 100110001


int16_t ShtTemper; 
uint16_t ShtHumi; 

typedef union{
	uint8_t u8[2];
	uint16_t u16;
	} SHT;
	
SHT ShT,ShHm;

uint8_t FlagSht;

//-------------------------------------------------------------------------------------------------
//-------------------------------------------------------------------------------------------------

static uint8_t SHT2x_CheckCrc(uint8_t data[], uint8_t nbrOfBytes, uint8_t checksum)
//==============================================================================
{
  //  CRC
  const uint16_t POLYNOMIAL = 0x131;  //P(x)=x^8+x^5+x^4+1 = 100110001
  uint8_t crc = 0;	
  uint8_t byteCtr;
  //calculates 8-Bit checksum with given polynomial
  for (byteCtr = 0; byteCtr < nbrOfBytes; ++byteCtr)
  { crc ^= (data[byteCtr]);
    for (uint8_t bit = 8; bit > 0; --bit)
    { if (crc & 0x80) crc = (crc << 1) ^ POLYNOMIAL;
      else crc = (crc << 1);
    }
  }
  if (crc != checksum) return 1;
  else return 0;
}

//--------------------------------------------------------------
void measure_sht21(void){//  1
static uint8_t State, Time1ms;
uint8_t error=0;

	switch(State){                     
		case (0):
			if (--Time1ms==0) State=1;
			break;
		
		case (1):// 
			error=i2c_Start();
			error|=i2c_wr_byte(I2C_ADR_W);
			error|=i2c_wr_byte(SOFT_RESET); 
			error|=i2c_Stop();
			Time1ms=30;
			State=2;
			break;
		
		case (2): 
			if (--Time1ms==0) 
				State=7;
				//State=3;//   .  3-5
			break;
/*		
		case (3): //   user reg
			error=i2c_Start();
			error=i2c_wr_byte(I2C_ADR_W); 
			error=i2c_wr_byte(USER_REG_R);
			State=4;
			break;	
		
		case (4)://
			i2c_Start();
			error=i2c_wr_byte(I2C_ADR_R);
			uint8_t r; 
			r=i2c_rx_byte(0); 
			if (r==0x3A) {
			// 7654 3210  
			// 0011 1010    . 8 9
				State=7; 
				i2c_Stop();
				}else 
				State=5;
			break;
		case (5): 
			i2c_Start();
			i2c_wr_byte(I2C_ADR_W); 
			i2c_wr_byte(USER_REG_W);
			i2c_wr_byte(0b00000010);
			i2c_Stop();
			error=1;
			break;
*/		
		case (7):
			if (FlagSht & FL_MEASURE_END)	break;//,     
			FlagSht|=FL_T_MEASURE;
			
		case (8): 
			error=i2c_Start();
			error|=i2c_wr_byte(I2C_ADR_W); 
			uint8_t Cmd=TRIG_RH_MEASUREMENT_POLL;
			if ( FlagSht & FL_T_MEASURE ) Cmd=TRIG_T_MEASUREMENT_POLL;
			error|=i2c_wr_byte(Cmd);
			error|=i2c_Stop();
			Time1ms=100;
			State=9;
			break;
		
		case (9): 
			if (--Time1ms==0) {
/*				error=i2c_Start();
				error=i2c_wr_byte(I2C_ADR_R); 
				if (error==0){
					State=10;
					}
					else{
					Time1ms=5;
					error=0;
					i2c_Stop();
					}*/
				State=10;	
				}
			break;
		
		case (10): 
			{
			uint8_t Buf[2], crc;
			error=i2c_Start();
			error|=i2c_wr_byte(I2C_ADR_R);
			Buf[0]=i2c_rx_byte(1);	//ASK
			Buf[1]=i2c_rx_byte(1);	//ASK
			crc =i2c_rx_byte(0);	//NO ASK
			error|=i2c_Stop();

			if ( FlagSht & FL_T_MEASURE ){ 
				FlagSht&=~FL_T_MEASURE;
				//if ( SHT2x_CheckCrc(Buf,2,crc)==0 ) FlagSht|=FL_T_REC; else FlagSht|=FL_T_CRC_ERR; 
				ShT.u8[0]=Buf[1];
				ShT.u8[1]=Buf[0];
				State=8; 
				}else{
				//if ( SHT2x_CheckCrc(Buf,2,crc)==0 ) FlagSht|=FL_HM_REC; else FlagSht|=FL_HM_CRC_ERR;	
				ShHm.u8[0]=Buf[1];
				ShHm.u8[1]=Buf[0];
				State=11;
				}
			}
			break;

		case (11): 
			{
			float Rez;
			uint16_t t=(int16_t)ShHm.u16;
			t&=~0x03;
			//-- calculate relative humidity [%RH]
			Rez = -60.0 + (1250.0/65536) * (float)t; // RH= -6 + 125 * SRH/2^16 
			ShtHumi=(uint16_t)Rez;
			}
			State=12;
			break;
		case (12): 
			{
			float Rez;
			int16_t t=(int16_t)ShT.u16;
			t&=~0x03;
			//-- calculate temperature [C]
			Rez = -468.5 + (1757.2/65536) * (float)t; //T= -46.85 + 175.72 * ST/2^16
			ShtTemper=(int16_t)Rez;
			}
			FlagSht|=FL_MEASURE_END;
			FlagSht&=~(FL_SHT_ERR|FL_NO_SENSOR);

			State=7;
			break;
			
		default : 
			Time1ms=20;
			State=0;
			break;	
		}
	
	if (error){
		FlagSht|=FL_SHT_ERR|FL_NO_SENSOR;
		State=0xFF;
		}
}
